Comment bouchonner en python (Stub)
===================================
![category](développement)
![tag](test)
![tag](python)

Il est parfois indispensable dans le cadre d'un test unitaire de "bouchonner"
une partie d'un programme

Prenons par exemple ce programme

```python
import os
import flask
from flask import request
import json
import mydomainsrvmetierlogin
import mydomainsrvmetierlogin.service as service

def load(app):

    @app.service("/metier/login", methods = ['GET','POST'])
    def login():
        """
            check login metier
            params:
                * login
                * password
        """
        serv = service.CheckLogin(login=request.args.get('login', ''),
                                    password=request.args.get('password', ''))
        if serv.check:
            return json.dumps({'msg':'welcome'})
        return  json.dumps({'msg':'bye bye'})

    @app.service("/metier/login/version", methods = ['GET','POST'])
    def getversionmydomainsrvmetierlogin():
        " retour la version du module mydomainsrvmetierlogin au format json"
        return  json.dumps({"version": mydomainsrvmetierlogin.__version__})
```

On utilise un module mydomainsrvmetierlogin.service qui est écrit ainsi

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from mydomainframeworkserv import ServiceWeb

class CheckLogin(ServiceWeb):

    url="http://127.0.0.1/myapp/com/checklogin"

    def __init__(self, login="", password=""):
        ServiceWeb.__init__(self)
        self._cl = self.get(params={'login':login,
                                    'password':password})
    @property
    def check(self):
        return self._cl.json()['check']
```

le soucis est que ce dernier utilise  un web service qui n'est pas encore en prod.
donc l'url http://127.0.0.1/myapp/com/checklogin ne répond pas.

Nous allons donc écrire un test "bouchonné"

```python
import unittest
from flask import Flask, request, json

import mydomainsrvmetierlogin.route
import mydomainsrvmetierlogin.service as service

class CheckLoginOk(object):

    def __init__(self, **kw):
        object.__init__(self)
        self.check = True

class CheckLoginKo(object):

    def __init__(self, **kw):
        object.__init__(self)
        self.check = False

class MyTest(unittest.TestCase):
    """
        Class for Unitaire Test for mydomainSrvMetierLogin
    """
    def setUp(self):
        self.app = Flask(__name__)
        self.app.testing = True
        self.app.service = self.app.route
        mydomainsrvmetierlogin.route.load(self.app)

    def test_login_ko(self):
        service.CheckLogin = CheckLoginKo
        with self.app.test_client() as c:
            rv = c.post('/metier/login')
            self.assertEqual(json.loads(rv.data)['msg'] , 'bye bye')

    def test_login_ok(self):
        service.CheckLogin = CheckLoginOk
        with self.app.test_client() as c:
            rv = c.post('/metier/login', query_string=dict(login='fred'))
            self.assertEqual(json.loads(rv.data)['msg'] , 'welcome')

if __name__ == '__main__':
    unittest.main()
```

le principe est de remplacer dynamiquement la classe odebosrvmetierlogin.service.CheckLogin
par une classe qui répond conformement à notre besoin (malgré que le webservice
que je souhaite utiliser n'existe pas)
